/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This file is part of SableCC.                             *
 * See the file "LICENSE" for copyright information and the  *
 * terms and conditions for copying, distribution and        *
 * modification of SableCC.                                  *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

package org.sablecc.sablecc;

import org.sablecc.sablecc.analysis.*;
import org.sablecc.sablecc.node.*;
import java.util.*;
import java.io.*;

/*
 * AddAstProductions
 * 
 * This class provide a part of the support of SableCC2.x.x grammars by 
 * SableCC3.x.x.
 * Its role is to add the section Abstract Syntax Tree and its productions
 * to the Grammar based on the Production Section.
 * It's the same result if a copy-paste of Productions section was added
 * to the SableCC2.x.x original grammar and renammed Abstract Syntax Tree.
 */

public class AddAstProductions extends DepthFirstAdapter
{

  List<PAstProd> listAstProd = new LinkedList<>();
  private boolean firstAlt;

  public AddAstProductions()
  {}

  @Override
  public void caseAProd(AProd node)
  {
    firstAlt = true;
    listOfAstAlts = new LinkedList<>();

    /*
     * Here, we assume that if there is no Abstract Syntax Tree Section specified
     * in the grammar, no transformations syntax is allowed in Productions section
     */
    if(node.getArrow() != null)
    {
      error(node.getArrow());
    }

    PAlt[] list_alt = node.getAlts().toArray(new PAlt[0]);
    for (PAlt alt : list_alt) {
      alt.apply(this);
    }

    AAstProd astProd = new AAstProd(new TId(node.getId().getText()), listOfAstAlts);
    listAstProd.add(astProd);
  }

  @Override
  public void outAGrammar(AGrammar node)
  {
    node.setAst(new AAst(listAstProd));
  }

  @Override
  public void inAAlt(AAlt node)
  {
    listElems = new LinkedList<>();
    processingParsedAlt = true;
  }

  @Override
  public void inAAltTransform(AAltTransform node)
  {
    if(node.getLBrace() != null)
    {
      error(node.getLBrace());
    }
  }

  @Override
  public void outAAlt(AAlt node)
  {
    TId aAltname = node.getAltName() == null ? null : node.getAltName().clone();
    AAstAlt astAlt = new AAstAlt(aAltname, listElems);

    listOfAstAlts.add(astAlt);
    processingParsedAlt = false;
  }

  boolean processingParsedAlt;

  @Override
  public void inAElem(AElem node)
  {
    if(processingParsedAlt)
    {
      listElems.add(node.clone());
    }
  }

  List<PElem> listElems;
  List<PAstAlt> listOfAstAlts;

  public void error(Token token)
  {
    throw new RuntimeException(
      "[" + token.getLine() + "," + token.getPos() + "] " +
      "AST transformations are not allowed because there are no section Abstract Syntax Tree");
  }
}
